#!/usr/env/python

#
# make_h_files.py - GnuLib.
#
# Python script to generate the needed .h-files in ./lib.
# Only for MinGW + MSVC + clang-cl.
#
#  By G. Vanem <gvanem@yahoo.no> 2012.

import sys, re, os, time

#
# Since MSVC doesn't support the '#include_next <>' statement,
# the next header MUST be included with an absolute path to
# the header!
#
# Not nice since the headers must be regenerated to be usable on
# another machine. Or when user updates to a newer 'Windows-Kit' version.
# Ref '$(WK_ROOT)' below.
#
vc_inc_dir = os.environ.get ('VCHOME')
vc_inc_dir += '/include'

#
# Hack for Microsoft's Universal CRT. Headers under 'Windows-Kit'.
# This requires 2 external environment variables to be set:
#   1) $(WK_ROOT) -> The root-directory of the 'Windows-Kit' installation.
#      E.g. do a 'set WK_ROOT=f:\ProgramFiles-x86\WindowsKits'.
#
#   2) $(WK_VER) -> The version of the installed 'WindowsKit'.
#      E.g. do a 'set WK_VER=10.0.10586.0'
#
# The directory structure for the 'Windows-Kit' seems to have changed some time
# during 2015. But for the location of the "WK" header-files we simply need:
#  $(WK_ROOT) + '\Include' + $(WK_VER) + '\ucrt'.
#
# E.g. 'f:/ProgramFiler-x86/WindowsKits/Include/10.0.10586.0/ucrt'
#       ========= $(WK_ROOT) ==========         $(WK_VER)
#
# Notes: the 'ucrt' part is the "User Mode CRT" header files.
#        'wk_inc_dir' MUST expand to the same as in ../Common.Windows.
#
#        to-do: 'check_vc_files()' should check the existance of this dir.
#        to-do: put this variable + test in 1 place only; on the cmdline
#               like for 'iconv_root'?
#

wk_root    = os.environ.get ('WK_ROOT')
wk_ver     = os.environ.get ('WK_VER')
wk_inc_dir = wk_root.replace('\\', '/') + '/include/' + wk_ver + '/ucrt'

h_file     = '??'
in_h_file  = '??'
iconv_root = '??'

where_i_am = os.getcwd()
who_am_i   = sys.argv[0]
now        = time.ctime()

def Fatal(s):
  print (s)
  sys.exit(1)

def check_file(f):
  if not os.path.exists(f):
    Fatal ("%s not found." % f)

def check_vc_files():
  ok_1 = os.path.exists (vc_inc_dir + '/stdlib.h')
  ok_2 = os.path.exists (wk_inc_dir + '/stdlib.h')
  if not ok_1 and not ok_2:
    Fatal ("%s nor %s found." % (vc_inc_dir + '/stdlib.h', wk_inc_dir + '/stdlib.h'))

def check_iconv_files():
  check_file (iconv_root  + '/include/iconv.h')
  check_file (iconv_root  + '/lib/iconv.c')

def slashify (file):
  if 0:
    if len(file) >= 2 and file[1] == ':':
      file[0].lower()
  file = os.path.normpath (file)
  return file.replace ('\\', '/')

def warning():
  return '''/*
 * DO NOT EDIT! This file was automatically generated by
 * %s/Makefile.Windows
 * (from %s and %s) at %s.
 * Edit those files instead.
 */

 ''' % (slashify(where_i_am), in_h_file, slashify(who_am_i), now)

def no_such_h_file():
  return 'include <no-such-file.h>'

def get_next_h (msvc_has_it = 1):
  if msvc_has_it == 0:
    return ('''if !defined(_MSC_VER) /* This "dead test" may seem funny, but the only way I figured to do it */
      #include_next <%s>
    #endif
    ''' % h_file)

  if os.path.exists(wk_inc_dir + '/' + h_file):
    h_dir = wk_inc_dir
  else:
    h_dir = vc_inc_dir

  return ('''if defined(_MSC_VER)
    #include "%s/%s"
  #else
    #include_next <%s>
  #endif
  ''' % (slashify(h_dir), h_file, h_file))

#
# <iconv.h> needs special handling.
#
def get_next_iconv_h():
  return ('''include "%s/include/iconv.h"
 /*
  * GnuLib must be rebuilt when this location changes
  */''' % iconv_root)

#
# <stdarg.h> needs special handling.
#
def get_next_stdarg_h():
  return ('''if defined(_MSC_VER)
      #if (_MSC_VER >= 1900)
        #include "%s/%s"
      #else
        #include "%s/%s"
      #endif
  #else
    #include_next <%s>
  #endif
  ''' % (slashify(h_dir), h_file, h_file))

#
# <inttypes.h> needs special handling.
# Replace this:
#   # @INCLUDE_NEXT@ @NEXT_INTTYPES_H@
# with:
#   #if defined(__MINGW32__)
#     #include_next <inttypes.h>
#   #elif defined(_MSC_VER) && (_MSC_VER >= 1800)
#     #include_next "$(wk_inc_dir)/inttypes.h"
#   #endif
#
def get_next_inttypes_h():
  return 'if ' + if_mingw() + '''
    #include_next <inttypes.h>
  #elif defined(_MSC_VER) && (_MSC_VER >= 1800)
    #include "%s/inttypes.h"
  #endif''' % slashify(wk_inc_dir)


def undef_wchar_min_max():
  return '''
#undef WCHAR_MIN  /* Redefined in our <stdint.h> */
#undef WCHAR_MAX  /* Ditto */
'''

def get_va_stuff():
  return ('''
/* Put this here since 'include_next <>' seems to have problems pulling
 * in the real gcc <stdarg.h>. Which in my case is:
 *   $MINGW32/lib/gcc/mingw32/4.7.2/include/stdarg.h
 */
#if !defined(va_list) && !defined(_ANSI_STDARG_H_) && defined(__GNUC__)
  #define va_list         __builtin_va_list
  #define va_start(v,l)   __builtin_va_start(v,l)
  #define va_end(v)       __builtin_va_end(v)
  #define va_arg(v,l)     __builtin_va_arg(v,l)
  #define __va_copy(d,s)  __builtin_va_copy(d,s)
  #define _VA_LIST_DEFINED
#endif
''')

def if_mingw (major = 0, minor = 0):
  if major == 0:
    return ('defined(__MINGW32__)')

  ver = r'(defined(__MINGW32__) && (__MINGW32_MAJOR_VERSION > %d || \
                                 (__MINGW32_MAJOR_VERSION == %d && \
                                  __MINGW32_MINOR_VERSION >= %d)) )'
  return ver % (major, major, minor)

def if_mingw64():
  return ('defined(__MINGW64_VERSION_MAJOR)')

def if_msvc (ver_higher_than = 0):
  if ver_higher_than == 0:
    return ('defined(_MSC_VER)')
  return ('(defined(_MSC_VER) && (_MSC_VER >= %d))' % ver_higher_than)

def if_mingw_or_msvc18():
  return ('( %s || %s )' % (if_mingw(), if_msvc(1800)))

def replace_iconv():
  return '1'  # if_mingw()

def make_h_file (d):
  def replace (match_obj):
    key = match_obj.group (1)
    try:
      return d[key]
    except KeyError:
      Fatal ("No substitution for @%s@ needed in %s. Edit 'config_dict()' as needed.\n" % (key, in_h_file))

  if 0:
    print ("Processing %s..." % in_fname)

  r1 = '/* The definitions of _GL_FUNCDECL_RPL etc. are copied here.  */'
  s1 = '#include <c++defs.h>'

  r2 = '/* The definition of _GL_WARN_ON_USE is copied here.  */'
  s2 = '#include <warn-on-use.h>'

  r3 = '/* The definition of _GL_ARG_NONNULL is copied here.  */'
  s3 = '#include <arg-nonnull.h>'

  r4 = '/* The definition of _Noreturn is copied here.  */'
  s4 = '#include <_Noreturn.h>'

  in_stat = os.stat (in_h_file)
  in_file = open (in_h_file)
  inp = in_file.read()

  out = re.sub (r"@([a-zA-Z0-9_]+)@", replace, inp)
  out = out.replace (r1, s1)
  out = out.replace (r2, s2)
  out = out.replace (r3, s3)
  out = out.replace (r4, s4)

  out_file = open (h_file, "w+")
  out_file.write (warning())
  out_file.write (out)
  out_file.close()

  #
  # 'touch' the .h-file to the time of .in-file.
  #
  if 0:
    os.utime (h_file, (in_stat.st_mtime, in_stat.st_mtime))

def show_help():
  print ('''Usage: %s: [-t] [-c] <.h-file> <.in.h-file> <ICONV_ROOT>"
  -t:           show dictonary of *all* keys and values.
  -c:           show additions for "config.h" file.
  <.h-file>:    the .h-file to generate.
  <.in.h-file>: the template to generate the .h-file from.
  <ICONV_ROOT>: root directory of your iconv-library.''' % who_am_i)
  return 0

#
# Handler for option '-t'.
#
def show_dict():

  def print_next_h (val, prefix = ""):
    p = val.find ("??")
    if p > 0:
      h_file = k [len(prefix)+5:].lower().replace ("_", ".")
      val = val[:p] + prefix + h_file + val[p+2:]
      p = val.find ("??")
      if p > 0:
        val = val[:p] + h_file + val[p+2:]
    if val == no_such_h_file():
      val = val + '\n'
    return "\n  #" + val

  d = config_dict()
  print ("%-45s %s\n%s" % ("Key", "value", "-"*52))
  for k in sorted(iter(d)):
    val = d[k]
    if k.startswith("NEXT_SYS_"):
      val = print_next_h (val, "sys/")
    elif k.startswith("NEXT_"):
      val = print_next_h (val)
    if val == "":
      val = "<none>"
    print ("%-45s = %s" % (k, val))
  return 0

#
# Handler for option '-c'.
#
def show_config():

  def print_have (key, val):
    #
    # val in ('0','1')
    #
    if len(val) == 1:
      print ("  #define %-40s %s" % (key, val))
    else:
      print ("  #if %s\n"            \
             "    #define %-38s 1\n" \
             "  #endif\n" % (val, key))

  d = config_dict()
  print ("  /*\n"
         "   * This list is generated from \"python.exe %s %s\"\n"
         "   * at %s. DO NOT EDIT!\n"
         "   */\n" % (__file__, sys.argv[1], now))

  for k in sorted(iter(d)):
    if k.startswith('GNULIB_'):
      print ("  #define %-40s %s" % (k, d[k]))
    elif k.startswith('HAVE_'):
      print_have (k, d[k])
  return 0

def main():
  if len(sys.argv) >= 2 and sys.argv[1] == '-t':
    sys.exit (show_dict())

  if len(sys.argv) >= 2 and sys.argv[1] == '-c':
    sys.exit (show_config())

  if (len(sys.argv) >= 2 and sys.argv[1] == '-h') or \
     len(sys.argv) != 4:
    sys.exit (show_help())

  check_vc_files()

  global h_file, in_h_file, iconv_root
  h_file     = sys.argv[1]
  in_h_file  = sys.argv[2]
  iconv_root = sys.argv[3]

  check_iconv_files()

  make_h_file (config_dict())

#
# This huge dictionary is used to replace '@FOO@' statements into
# sensible values in the generated .h-files.
# The "d['HAVE_*']" and "d['GNULIB_*']" dict-values are also used to generate the ./config.h file.
#
def config_dict():
  d = {}
  d['APPLE_UNIVERSAL_BUILD']                       = '0'
  d['ASM_SYMBOL_PREFIX']                           = '"_"'
  d['EMULTIHOP_HIDDEN']                            = '0'
  d['EMULTIHOP_VALUE']                             = '0'
  d['ENOLINK_HIDDEN']                              = '0'
  d['ENOLINK_VALUE']                               = '0'
  d['EOVERFLOW_HIDDEN']                            = '0'
  d['EOVERFLOW_VALUE']                             = '0'

  #
  # <stdint.h> stuff:
  #
  d['BITSIZEOF_PTRDIFF_T']                         = '32'
  d['PTRDIFF_T_SUFFIX']                            = ''
  d['SIG_ATOMIC_MIN']                              = 'INTPTR_MIN'
  d['BITSIZEOF_SIG_ATOMIC_T']                      = '32'
  d['BITSIZEOF_SIZE_T']                            = '32'
  d['BITSIZEOF_WCHAR_T']                           = '16'
  d['BITSIZEOF_WINT_T']                            = '16'
  d['HAVE_SIGNED_SIG_ATOMIC_T']                    = '1'
  d['HAVE_SIGNED_WCHAR_T']                         = '0'
  d['HAVE_SIGNED_WINT_T']                          = '0'
  d['SIG_ATOMIC_T_SUFFIX']                         = ''
  d['SIZE_T_SUFFIX']                               = 'I64'
  d['WCHAR_T_SUFFIX']                              = ''
  d['WINT_T_SUFFIX']                               = ''

  d['GNULIB_FDUTIMENSAT']                          = '1'
  d['GNULIB_FNMATCH']                              = '1'
  d['GNULIB_FWRITEERROR']                          = '1'
  d['GNULIB_CLOSE_STREAM']                         = '1'
  d['GNULIB_CANONICALIZE_LGPL']                    = '1'
  d['GNULIB_DIRNAME']                              = '1'
  d['GNULIB_SIGPIPE']                              = '1'
  d['GNULIB_STATAT']                               = '1'
  d['GNULIB_PROGRAM_INVOCATION_NAME']              = '1'
  d['GNULIB_NAMESPACE']                            = 'gnulib'
  d['GNULIB_GC_MD2']                               = '1'
  d['GNULIB_GC_MD4']                               = '1'
  d['GNULIB_GC_MD5']                               = '1'
  d['GNULIB_GC_SHA1']                              = '1'
  d['GNULIB_GC_HMAC_MD5']                          = '1'
  d['GNULIB_GC_HMAC_SHA1']                         = '1'
  d['GNULIB_GC_HMAC_SHA256']                       = '1'
  d['GNULIB_GC_HMAC_SHA512']                       = '1'
  d['GNULIB_GC_ARCFOUR']                           = '1'
  d['GNULIB_GC_ARCTWO']                            = '1'
  d['GNULIB_GC_DES']                               = '1'
  d['GNULIB_GC_RIJNDAEL']                          = '1'
  d['GNULIB_GC_RANDOM']                            = '1'
  d['GNULIB_GLOB']                                 = '1'
  d['GNULIB_ACCEPT4']                              = '1'
  d['GNULIB_ACCEPT']                               = '1'
  d['GNULIB_ACOSF']                                = '!' + if_msvc(1800)
  d['GNULIB_ACOSL']                                = if_msvc()
  d['GNULIB_ALPHASORT']                            = '1'
  d['GNULIB_ASINF']                                = if_msvc()
  d['GNULIB_ASINL']                                = if_msvc()
  d['GNULIB_ATAN2F']                               = if_msvc()
  d['GNULIB_ATANF']                                = if_msvc()
  d['GNULIB_ATANL']                                = if_msvc()
  d['GNULIB_ATOLL']                                = '1'
  d['GNULIB_AREADLINKAT']                          = '1'
  d['GNULIB_AREADLINKAT_WITH_SIZE']                = '1'
  d['GNULIB_BIND']                                 = '1'
  d['GNULIB_BTOWC']                                = '1'
  d['GNULIB_CALLOC_POSIX']                         = '1'
  d['GNULIB_CANONICALIZE_FILE_NAME']               = '1'
  d['GNULIB_CBRT']                                 = '0'
  d['GNULIB_CBRTF']                                = '0'
  d['GNULIB_CBRTL']                                = '0'
  d['GNULIB_CEIL']                                 = '1'
  d['GNULIB_CEILF']                                = '1'
  d['GNULIB_CEILL']                                = '1'
  d['GNULIB_CHDIR']                                = '1'
  d['GNULIB_CHOWN']                                = '1'
  d['GNULIB_CLOSE']                                = '1'
  d['GNULIB_CLOSEDIR']                             = '1'
  d['GNULIB_CONNECT']                              = '1'
  d['GNULIB_COPYSIGN']                             = '1'
  d['GNULIB_COPYSIGNF']                            = '1'
  d['GNULIB_COPYSIGNL']                            = '1'
  d['GNULIB_COSF']                                 = '1'
  d['GNULIB_COSHF']                                = '1'
  d['GNULIB_COSL']                                 = '1'
  d['GNULIB_CTIME']                                = '1'
  d['GNULIB_DIRFD']                                = '1'
  d['GNULIB_DPRINTF']                              = '1'
  d['GNULIB_DUP2']                                 = '1'
  d['GNULIB_DUP3']                                 = '1'
  d['GNULIB_DUP']                                  = '1'
  d['GNULIB_DUPLOCALE']                            = '0'
  d['GNULIB_ENVIRON']                              = '1'
  d['GNULIB_EUIDACCESS']                           = '1'
  d['GNULIB_EXP2']                                 = '0'
  d['GNULIB_EXP2F']                                = '0'
  d['GNULIB_EXP2L']                                = '0'
  d['GNULIB_EXPF']                                 = '0'
  d['GNULIB_EXPL']                                 = '0'
  d['GNULIB_EXPM1']                                = '0'
  d['GNULIB_EXPM1F']                               = '0'
  d['GNULIB_EXPM1L']                               = '0'
  d['GNULIB_EXPLICIT_BZERO']                       = '1'
  d['GNULIB_FABSF']                                = '0'
  d['GNULIB_FABSL']                                = '0'
  d['GNULIB_FACCESSAT']                            = '1'
  d['GNULIB_FCHDIR']                               = '1'
  d['GNULIB_FCHMODAT']                             = '1'
  d['GNULIB_FCHOWNAT']                             = '1'
  d['GNULIB_FCLOSE']                               = '1'  # '!' + if_msvc(1800)
  d['GNULIB_FCNTL']                                = '1'
  d['GNULIB_FDATASYNC']                            = '1'
  d['GNULIB_FDOPEN']                               = '1'
  d['GNULIB_FDOPENDIR']                            = '1'
  d['GNULIB_FFLUSH']                               = '1'  # '!' + if_msvc(1800)
  d['GNULIB_FFS']                                  = '1'
  d['GNULIB_FFSL']                                 = '1'
  d['GNULIB_FFSLL']                                = '1'
  d['GNULIB_FGETC']                                = '1'
  d['GNULIB_FGETS']                                = '1'
  d['GNULIB_FLOCK']                                = '1'
  d['GNULIB_FLOOR']                                = '0'
  d['GNULIB_FLOORF']                               = '0'
  d['GNULIB_FLOORL']                               = '0'
  d['GNULIB_FMA']                                  = '1'
  d['GNULIB_FMAF']                                 = '1'
  d['GNULIB_FMAL']                                 = '1'
  d['GNULIB_FMOD']                                 = '0'
  d['GNULIB_FMODF']                                = '0'
  d['GNULIB_FMODL']                                = '0'
  d['GNULIB_FOPEN']                                = '1'
  d['GNULIB_FORKPTY']                              = '1'
  d['GNULIB_FPRINTF']                              = '1'
  d['GNULIB_FPRINTF_POSIX']                        = '1'
  d['GNULIB_FPURGE']                               = '1'
  d['GNULIB_FPUTC']                                = '1'
  d['GNULIB_FPUTS']                                = '1'
  d['GNULIB_FREAD']                                = '1'
  d['GNULIB_FREOPEN']                              = '1'
  d['GNULIB_FREXP']                                = '0'
  d['GNULIB_FREXPF']                               = '0'
  d['GNULIB_FREXPL']                               = '0'
  d['GNULIB_FSCANF']                               = '0'
  d['GNULIB_FSEEK']                                = '1'  # if_mingw_or_msvc18()
  d['GNULIB_FSEEKO']                               = '1'  # if_mingw_or_msvc18()
  d['GNULIB_FSTAT']                                = '1'
  d['GNULIB_FSTATAT']                              = '1'
  d['GNULIB_FSYNC']                                = '1'
  d['GNULIB_FTELL']                                = '1'
  d['GNULIB_FTELLO']                               = '1'
  d['GNULIB_FTRUNCATE']                            = '1'
  d['GNULIB_FUTIMENS']                             = '1'
  d['GNULIB_FWRITE']                               = '1'
  d['GNULIB_GETADDRINFO']                          = '1'
  d['GNULIB_GETC']                                 = '1'
  d['GNULIB_GETCHAR']                              = '1'
  d['GNULIB_GETCWD']                               = '0'
  d['GNULIB_GETDELIM']                             = '1'
  d['GNULIB_GETDOMAINNAME']                        = '1'
  d['GNULIB_GETDTABLESIZE']                        = '1'
  d['GNULIB_GETGROUPS']                            = '1'
  d['GNULIB_GETHOSTNAME']                          = '1'
  d['GNULIB_GETLINE']                              = '1'
  d['GNULIB_GETLOADAVG']                           = '1'
  d['GNULIB_GETLOGIN']                             = '1'
  d['GNULIB_GETLOGIN_R']                           = '1'
  d['GNULIB_GETPAGESIZE']                          = '1'
  d['GNULIB_GETPASS']                              = '1'
  d['GNULIB_GETPEERNAME']                          = '1'
  d['GNULIB_GETRUSAGE']                            = '1'
  d['GNULIB_GETSOCKNAME']                          = '1'
  d['GNULIB_GETSOCKOPT']                           = '1'
  d['GNULIB_GETSUBOPT']                            = '1'
  d['GNULIB_GETTIMEOFDAY']                         = '1'
  d['GNULIB_GETUSERSHELL']                         = '1'
  d['GNULIB_GRANTPT']                              = '1'
  d['GNULIB_GROUP_MEMBER']                         = '1'
  d['GNULIB_HYPOT']                                = '0'
  d['GNULIB_HYPOTF']                               = '0'
  d['GNULIB_HYPOTL']                               = '0'
  d['GNULIB_ICONV']                                = replace_iconv()
  d['GNULIB_ILOGB']                                = '1'
  d['GNULIB_ILOGBF']                               = '1'
  d['GNULIB_ILOGBL']                               = '1'
  d['GNULIB_IMAXABS']                              = '0'
  d['GNULIB_IMAXDIV']                              = '0'
  d['GNULIB_INET_NTOP']                            = '1'
  d['GNULIB_INET_PTON']                            = '1'
  d['GNULIB_IOCTL']                                = '1'
  d['GNULIB_ISATTY']                               = '1'
  d['GNULIB_ISBLANK']                              = '1'
  d['GNULIB_ISFINITE']                             = '!' + if_msvc(1800)
  d['GNULIB_ISINF']                                = '1'
  d['GNULIB_ISNAN']                                = '1'
  d['GNULIB_ISNAND']                               = '1'
  d['GNULIB_ISNANF']                               = '1'
  d['GNULIB_ISNANL']                               = '1'
  d['GNULIB_ISWBLANK']                             = '1'
  d['GNULIB_ISWCTYPE']                             = '1'
  d['GNULIB_LCHMOD']                               = '1'
  d['GNULIB_LCHOWN']                               = '1'
  d['GNULIB_LDEXPF']                               = '0'
  d['GNULIB_LDEXPL']                               = '0'
  d['GNULIB_LINK']                                 = '1'
  d['GNULIB_LINKAT']                               = '1'
  d['GNULIB_LISTEN']                               = '1'
  d['GNULIB_LOCALECONV']                           = '1'
  d['GNULIB_LOCALTIME']                           = '1'
  d['GNULIB_LOG10']                                = '0'
  d['GNULIB_LOG10F']                               = '0'
  d['GNULIB_LOG10L']                               = '0'
  d['GNULIB_LOG1P']                                = '1'
  d['GNULIB_LOG1PF']                               = '1'
  d['GNULIB_LOG1PL']                               = '1'
  d['GNULIB_LOG2']                                 = '1'
  d['GNULIB_LOG2F']                                = '1'
  d['GNULIB_LOG2L']                                = '1'
  d['GNULIB_LOG']                                  = '0'
  d['GNULIB_LOGB']                                 = '1'
  d['GNULIB_LOGBF']                                = '1'
  d['GNULIB_LOGBL']                                = '1'
  d['GNULIB_LOGF']                                 = '0'
  d['GNULIB_LOGL']                                 = '0'
  d['GNULIB_LSEEK']                                = '1'
  d['GNULIB_LSTAT']                                = '1'
  d['GNULIB_MALLOC_POSIX']                         = '1'
  d['GNULIB_MBRLEN']                               = '1'
  d['GNULIB_MBRTOWC']                              = '1'
  d['GNULIB_MBSCASECMP']                           = '1'
  d['GNULIB_MBSCASESTR']                           = '1'
  d['GNULIB_MBSCHR']                               = '1'
  d['GNULIB_MBSCSPN']                              = '1'
  d['GNULIB_MBSINIT']                              = '!' + if_msvc(1900)
  d['GNULIB_MBSLEN']                               = '1'
  d['GNULIB_MBSNCASECMP']                          = '1'
  d['GNULIB_MBSNLEN']                              = '1'
  d['GNULIB_MBSNRTOWCS']                           = '1'
  d['GNULIB_MBSPBRK']                              = '1'
  d['GNULIB_MBSPCASECMP']                          = '1'
  d['GNULIB_MBSRCHR']                              = '1'
  d['GNULIB_MBSRTOWCS']                            = '1'
  d['GNULIB_MBSSEP']                               = '1'
  d['GNULIB_MBSSPN']                               = '1'
  d['GNULIB_MBSSTR']                               = '1'
  d['GNULIB_MBSTOK_R']                             = '1'
  d['GNULIB_MBTOWC']                               = '1'
  d['GNULIB_MEMCHR']                               = '1'
  d['GNULIB_MEMMEM']                               = '1'
  d['GNULIB_MEMPCPY']                              = '1'
  d['GNULIB_MEMRCHR']                              = '1'
  d['GNULIB_MKDIRAT']                              = '1'
  d['GNULIB_MKDTEMP']                              = '1'
  d['GNULIB_MKFIFO']                               = '1'
  d['GNULIB_MKFIFOAT']                             = '1'
  d['GNULIB_MKNOD']                                = '1'
  d['GNULIB_MKNODAT']                              = '1'
  d['GNULIB_MKOSTEMP']                             = '1'
  d['GNULIB_MKOSTEMPS']                            = '1'
  d['GNULIB_MKSTEMP']                              = '1'
  d['GNULIB_MKSTEMPS']                             = '1'
  d['GNULIB_MKTIME']                               = '1'
  d['GNULIB_MODF']                                 = '0'
  d['GNULIB_MODFF']                                = '0'
  d['GNULIB_MODFL']                                = '0'
  d['GNULIB_NANOSLEEP']                            = '!' + if_mingw64()
  d['GNULIB_NL_LANGINFO']                          = '1'
  d['GNULIB_NONBLOCKING']                          = '1'
  d['GNULIB_OBSTACK_PRINTF']                       = '1'
  d['GNULIB_OBSTACK_PRINTF_POSIX']                 = '1'
  d['GNULIB_OPEN']                                 = '1'
  d['GNULIB_OPENAT']                               = '1'
  d['GNULIB_OPENDIR']                              = '1'
  d['GNULIB_OPENPTY']                              = '1'
  d['GNULIB_OVERRIDES_WINT_T']                     = '1'
  d['GNULIB_OVERRIDES_STRUCT_STAT']                = '1'
  d['GNULIB_PCLOSE']                               = '1'
  d['GNULIB_PERROR']                               = '1'
  d['GNULIB_PIPE2']                                = '1'
  d['GNULIB_PIPE']                                 = '1'
  d['GNULIB_POLL']                                 = '1'
  d['GNULIB_POPEN']                                = '1'
  d['GNULIB_POSIX_OPENPT']                         = '1'
  d['GNULIB_POSIX_SPAWN']                          = '1'
  d['GNULIB_POSIX_SPAWNATTR_DESTROY']              = '1'
  d['GNULIB_POSIX_SPAWNATTR_GETFLAGS']             = '1'
  d['GNULIB_POSIX_SPAWNATTR_GETPGROUP']            = '1'
  d['GNULIB_POSIX_SPAWNATTR_GETSCHEDPARAM']        = '1'
  d['GNULIB_POSIX_SPAWNATTR_GETSCHEDPOLICY']       = '1'
  d['GNULIB_POSIX_SPAWNATTR_GETSIGDEFAULT']        = '1'
  d['GNULIB_POSIX_SPAWNATTR_GETSIGMASK']           = '1'
  d['GNULIB_POSIX_SPAWNATTR_INIT']                 = '1'
  d['GNULIB_POSIX_SPAWNATTR_SETFLAGS']             = '1'
  d['GNULIB_POSIX_SPAWNATTR_SETPGROUP']            = '1'
  d['GNULIB_POSIX_SPAWNATTR_SETSCHEDPARAM']        = '1'
  d['GNULIB_POSIX_SPAWNATTR_SETSCHEDPOLICY']       = '1'
  d['GNULIB_POSIX_SPAWNATTR_SETSIGDEFAULT']        = '1'
  d['GNULIB_POSIX_SPAWNATTR_SETSIGMASK']           = '1'
  d['GNULIB_POSIX_SPAWNP']                         = '1'
  d['GNULIB_POSIX_SPAWN_FILE_ACTIONS_ADDCHDIR']    = '1'
  d['GNULIB_POSIX_SPAWN_FILE_ACTIONS_ADDCLOSE']    = '1'
  d['GNULIB_POSIX_SPAWN_FILE_ACTIONS_ADDDUP2']     = '1'
  d['GNULIB_POSIX_SPAWN_FILE_ACTIONS_ADDOPEN']     = '1'
  d['GNULIB_POSIX_SPAWN_FILE_ACTIONS_DESTROY']     = '1'
  d['GNULIB_POSIX_SPAWN_FILE_ACTIONS_INIT']        = '1'
  d['GNULIB_POWF']                                 = '0'
  d['GNULIB_PREAD']                                = '1'
  d['GNULIB_PRINTF']                               = '1'
  d['GNULIB_PRINTF_POSIX']                         = '1'
  d['GNULIB_PSELECT']                              = '1'
  d['GNULIB_PTHREAD_SIGMASK']                      = '!' + if_mingw64()
  d['GNULIB_PTSNAME']                              = '1'
  d['GNULIB_PTSNAME_R']                            = '1'
  d['GNULIB_PUTC']                                 = '1'
  d['GNULIB_PUTCHAR']                              = '1'
  d['GNULIB_PUTENV']                               = '1'
  d['GNULIB_PUTS']                                 = '1'
  d['GNULIB_PWRITE']                               = '1'
  d['GNULIB_QSORT_R']                              = '1'
  d['GNULIB_RAISE']                                = '1'
  d['GNULIB_RANDOM']                               = '1'
  d['GNULIB_RANDOM_R']                             = '1'
  d['GNULIB_RAWMEMCHR']                            = '1'
  d['GNULIB_READ']                                 = '1'
  d['GNULIB_READDIR']                              = '1'
  d['GNULIB_READLINK']                             = '1'
  d['GNULIB_READLINKAT']                           = '1'
  d['GNULIB_REALLOC_POSIX']                        = '1'
  d['GNULIB_REALLOCARRAY']                         = '1'
  d['GNULIB_REALPATH']                             = '1'
  d['GNULIB_RECV']                                 = '1'
  d['GNULIB_RECVFROM']                             = '1'
  d['GNULIB_REMAINDER']                            = '1'
  d['GNULIB_REMAINDERF']                           = '1'
  d['GNULIB_REMAINDERL']                           = '1'
  d['GNULIB_REMOVE']                               = '!' + if_msvc(1900)
  d['GNULIB_RENAME']                               = '1'
  d['GNULIB_RENAMEAT']                             = '1'
  d['GNULIB_REWINDDIR']                            = '1'
  d['GNULIB_RINT']                                 = '1'
  d['GNULIB_RINTF']                                = '1'
  d['GNULIB_RINTL']                                = '1'
  d['GNULIB_RMDIR']                                = '1'
  d['GNULIB_ROUND']                                = '1'
  d['GNULIB_ROUNDF']                               = '1'
  d['GNULIB_ROUNDL']                               = '1'
  d['GNULIB_RPMATCH']                              = '1'
  d['GNULIB_SCANDIR']                              = '1'
  d['GNULIB_SCANF']                                = '0'
  d['GNULIB_SECURE_GETENV']                        = '1'
  d['GNULIB_SELECT']                               = '1'
  d['GNULIB_SEND']                                 = '1'
  d['GNULIB_SENDTO']                               = '1'
  d['GNULIB_SETENV']                               = '1'
  d['GNULIB_SETHOSTNAME']                          = '1'
  d['GNULIB_SETLOCALE']                            = '1'
  d['GNULIB_SETSOCKOPT']                           = '1'
  d['GNULIB_SHUTDOWN']                             = '1'
  d['GNULIB_SIGACTION']                            = '1'
  d['GNULIB_SIGNAL_H_SIGPIPE']                     = '1'
  d['GNULIB_SIGNBIT']                              = '1'
  d['GNULIB_SIGPROCMASK']                          = '1'
  d['GNULIB_SINF']                                 = '0'
  d['GNULIB_SINHF']                                = '0'
  d['GNULIB_SINL']                                 = '0'
  d['GNULIB_SLEEP']                                = '1'
  d['GNULIB_SNPRINTF']                             = '1'
  d['GNULIB_SOCKET']                               = '1'
  d['GNULIB_SPRINTF_POSIX']                        = '1'
  d['GNULIB_SQRTF']                                = '0'
  d['GNULIB_SQRTL']                                = '0'
  d['GNULIB_STAT']                                 = '1'
  d['GNULIB_STDIO_H_NONBLOCKING']                  = '1'
  d['GNULIB_STDIO_H_SIGPIPE']                      = '1'
  d['GNULIB_STPCPY']                               = '1'
  d['GNULIB_STPNCPY']                              = '1'
  d['GNULIB_STRCASESTR']                           = '1'
  d['GNULIB_STRCHRNUL']                            = '1'
  d['GNULIB_STRDUP']                               = '0'
  d['GNULIB_STRERROR']                             = '1'
  d['GNULIB_STRERROR_R']                           = '1'
  d['GNULIB_STRFMON_L']                            = '1'
  d['GNULIB_STRNCAT']                              = '0'
  d['GNULIB_STRNDUP']                              = '1'
  d['GNULIB_STRNLEN']                              = '1'
  d['GNULIB_STRPBRK']                              = '0'
  d['GNULIB_STRPTIME']                             = '1'
  d['GNULIB_STRFTIME']                             = '1'
  d['GNULIB_STRSEP']                               = '1'
  d['GNULIB_STRSIGNAL']                            = '1'
  d['GNULIB_STRSTR']                               = '0'
  d['GNULIB_STRTOD']                               = '1'
  d['GNULIB_STRTOK_R']                             = '1'
  d['GNULIB_STRTOLL']                              = '1'
  d['GNULIB_STRTOULL']                             = '1'
  d['GNULIB_STRTOIMAX']                            = '1'
  d['GNULIB_STRTOUMAX']                            = '1'
  d['GNULIB_STRVERSCMP']                           = '1'
  d['GNULIB_SYMLINK']                              = '1'
  d['GNULIB_SYMLINKAT']                            = '1'
  d['GNULIB_SYSTEM_POSIX']                         = '1'
  d['GNULIB_TANF']                                 = '0'
  d['GNULIB_TANHF']                                = '0'
  d['GNULIB_TANL']                                 = '0'
  d['GNULIB_TCGETSID']                             = '1'
  d['GNULIB_TIMEGM']                               = '1'
  d['GNULIB_TIMES']                                = '1'
  d['GNULIB_TIME_R']                               = '1'
  d['GNULIB_TIME_RZ']                              = '1'
  d['GNULIB_TMPFILE']                              = '1'
  d['GNULIB_TZSET']                                = '1'
  d['GNULIB_TOWCTRANS']                            = '0'
  d['GNULIB_TRUNC']                                = '0'
  d['GNULIB_TRUNCATE']                             = '1'
  d['GNULIB_TRUNCF']                               = '0'
  d['GNULIB_TRUNCL']                               = '0'
  d['GNULIB_TSEARCH']                              = '1'
  d['GNULIB_TTYNAME_R']                            = '1'
  d['GNULIB_UNAME']                                = '1'
  d['GNULIB_UNISTD_H_GETOPT']                      = '1'
  d['GNULIB_UNLINK']                               = '1'
  d['GNULIB_UNLINKAT']                             = '1'
  d['GNULIB_UNLOCKPT']                             = '1'
  d['GNULIB_UNSETENV']                             = '1'
  d['GNULIB_USLEEP']                               = '1'
  d['GNULIB_UTIME']                                = '1'
  d['GNULIB_UTIMENSAT']                            = '1'
  d['GNULIB_VASPRINTF']                            = '1'
  d['GNULIB_VDPRINTF']                             = '1'
  d['GNULIB_VFPRINTF']                             = '1'
  d['GNULIB_VFPRINTF_POSIX']                       = '1'
  d['GNULIB_VFSCANF']                              = if_mingw_or_msvc18()
  d['GNULIB_VPRINTF']                              = '1'
  d['GNULIB_VPRINTF_POSIX']                        = '1'
  d['GNULIB_VSCANF']                               = '0'
  d['GNULIB_VSNPRINTF']                            = '1'
  d['GNULIB_VSPRINTF_POSIX']                       = '1'
  d['GNULIB_WAITPID']                              = '1'
  d['GNULIB_WCPCPY']                               = '1'
  d['GNULIB_WCPNCPY']                              = '1'
  d['GNULIB_WCRTOMB']                              = '1'
  d['GNULIB_WCSCASECMP']                           = '1'
  d['GNULIB_WCSCAT']                               = '1'
  d['GNULIB_WCSCHR']                               = '1'
  d['GNULIB_WCSCMP']                               = '1'
  d['GNULIB_WCSCOLL']                              = '1'
  d['GNULIB_WCSCPY']                               = if_mingw()
  d['GNULIB_WCSCSPN']                              = '1'
  d['GNULIB_WCSDUP']                               = '1'
  d['GNULIB_WCSLEN']                               = '1'
  d['GNULIB_WCSNCASECMP']                          = '1'
  d['GNULIB_WCSNCAT']                              = '1'
  d['GNULIB_WCSNCMP']                              = '1'
  d['GNULIB_WCSFTIME']                             = '1'
  d['GNULIB_WCSNCPY']                              = '1'
  d['GNULIB_WCSNLEN']                              = '1'
  d['GNULIB_WCSNRTOMBS']                           = '1'
  d['GNULIB_WCSPBRK']                              = '1'
  d['GNULIB_WCSRCHR']                              = '1'
  d['GNULIB_WCSRTOMBS']                            = '1'
  d['GNULIB_WCSSPN']                               = '1'
  d['GNULIB_WCSSTR']                               = '1'
  d['GNULIB_WCSTOK']                               = '1'
  d['GNULIB_WCSWIDTH']                             = '1'
  d['GNULIB_WCSXFRM']                              = '1'
  d['GNULIB_WCTOB']                                = '1'
  d['GNULIB_WCTOMB']                               = '1'
  d['GNULIB_WCTRANS']                              = '0'
  d['GNULIB_WCTYPE']                               = '0'
  d['GNULIB_WCWIDTH']                              = '1'
  d['GNULIB_WMEMCHR']                              = '1'
  d['GNULIB_WMEMCMP']                              = '1'
  d['GNULIB_WMEMCPY']                              = '1'
  d['GNULIB_WMEMMOVE']                             = '1'
  d['GNULIB_WMEMSET']                              = '1'
  d['GNULIB_WRITE']                                = '1'
  d['GNULIB__EXIT']                                = '!' + if_msvc(1900)
  d['GUARD_PREFIX']                                = 'GNULIB'

  d['GNULIB_TEST_BTOWC']                           = '1'
  d['GNULIB_TEST_ENVIRON']                         = '1'
  d['GNULIB_TEST_FDOPEN']                          = '1'
  d['GNULIB_TEST_FDOPENDIR']                       = '1'
  d['GNULIB_TEST_FREXP']                           = '1'
  d['GNULIB_TEST_FREXPL']                          = '1'
  d['GNULIB_TEST_GETPAGESIZE']                     = '1'
  d['GNULIB_TEST_GETTIMEOFDAY']                    = '1'
  d['GNULIB_TEST_ISWBLANK']                        = '1'
  d['GNULIB_TEST_MALLOC_POSIX']                    = '1'
  d['GNULIB_TEST_MBRTOWC']                         = '1'
  d['GNULIB_TEST_MBSINIT']                         = '1'
  d['GNULIB_TEST_MBSNLEN']                         = '1'
  d['GNULIB_TEST_MBTOWC']                          = '1'
  d['GNULIB_TEST_MEMCHR']                          = '1'
  d['GNULIB_TEST_PUTENV']                          = '1'
  d['GNULIB_TEST_SETENV']                          = '1'
  d['GNULIB_TEST_SETLOCALE']                       = '1'
  d['GNULIB_TEST_SIGNBIT']                         = '1'
  d['GNULIB_TEST_SLEEP']                           = '1'
  d['GNULIB_TEST_STRERROR']                        = '1'
  d['GNULIB_TEST_UNSETENV']                        = '1'
  d['GNULIB_TEST_USLEEP']                          = '1'
  d['GNULIB_TEST_WCRTOMB']                         = '1'
  d['GNULIB_TEST_WCTOB']                           = '1'
  d['GNULIB_TEST_WCTOMB']                          = '1'
  d['GNULIB_TEST_WCWIDTH']                         = '1'

  d['HAVE_ACCEPT4']                                = '0'
  d['HAVE_SAME_LONG_DOUBLE_AS_DOUBLE']             = '1'
  d['HAVE_SEARCH_H']                               = '1'
  d['HAVE_INLINE']                                 = '0'
  d['HAVE_IPV4']                                   = '1'
  d['HAVE_IPV6']                                   = '1'
  d['HAVE_ALLOCA']                                 = '1'
  d['HAVE_QSORT_R']                                = '0'
  d['HAVE_RAW_DECL_EXPLICIT_BZERO']                = '0'
  d['HAVE_RAW_DECL_GETHOSTNAME']                   = '1'
  d['HAVE_RAW_DECL_SETHOSTNAME']                   = '0'
  d['HAVE_RAW_DECL_LOCALECONV']                    = '1'
  d['HAVE_RAW_DECL_SETLOCALE']                     = '1'
  d['HAVE_RAW_DECL_DUPLOCALE']                     = '0'
  d['HAVE_RAW_DECL_GETLOADAVG']                    = '0'
  d['HAVE_RAW_DECL_ISWCTYPE']                      = '1'
  d['HAVE_RAW_DECL_REALLOCARRAY']                  = '0'
  d['HAVE_RAW_DECL_REWINDDIR']                     = if_mingw()
  d['HAVE_RAW_DECL_STRTOIMAX']                     = if_mingw_or_msvc18()
  d['HAVE_RAW_DECL_STRTOUMAX']                     = if_mingw_or_msvc18()
  d['HAVE_RAW_DECL_STRFMON_L']                     = '0'
  d['HAVE_RAW_DECL_ILOGB']                         = if_mingw()
  d['HAVE_RAW_DECL_ILOGBF']                        = if_mingw()
  d['HAVE_RAW_DECL_ILOGBL']                        = if_mingw()
  d['HAVE_RAW_DECL_PCLOSE']                        = if_mingw()
  d['HAVE_RAW_DECL_WCWIDTH']                       = '0'
  d['HAVE_RAW_DECL_WMEMCHR']                       = '1'
  d['HAVE_RAW_DECL_WMEMCMP']                       = '1'
  d['HAVE_RAW_DECL_WMEMCPY']                       = '1'
  d['HAVE_DECL___ARGV']                            = '1'
  d['HAVE_DECL_PUTC_UNLOCKED']                     = '0'
  d['HAVE_DECL_CLEARERR_UNLOCKED']                 = '0'
  d['HAVE_DECL_FEOF_UNLOCKED']                     = '0'
  d['HAVE_DECL_FERROR_UNLOCKED']                   = '0'
  d['HAVE_DECL_FFLUSH_UNLOCKED']                   = '0'
  d['HAVE_DECL_FPUTC_UNLOCKED']                    = '0'
  d['HAVE_DECL_FPUTS_UNLOCKED']                    = '0'
  d['HAVE_DECL_FREAD_UNLOCKED']                    = '0'
  d['HAVE_DECL_FWRITE_UNLOCKED']                   = '0'
  d['HAVE_DECL_GETC_UNLOCKED']                     = '0'
  d['HAVE_DECL_GETCHAR_UNLOCKED']                  = '0'
  d['HAVE_DECL_PUTC_UNLOCKED']                     = '0'
  d['HAVE_DECL__PUTENV']                           = '1'
  d['HAVE_DECL_PUTCHAR_UNLOCKED']                  = '0'
  d['HAVE_DECL_STRERROR_R']                        = '0'
  d['HAVE_DECL_PROGRAM_INVOCATION_NAME']           = '0'
  d['HAVE_DECL_STRTOLL']                           = '1'
  d['HAVE_DECL_STRTOULL']                          = '1'
  d['HAVE_STDINT_H']                               = '1'
  d['HAVE_GETCWD']                                 = '1'
  d['HAVE_GLOB']                                   = '0'
  d['HAVE_GLOB_PATTERN_P']                         = '0'
  d['HAVE_C99_STDINT_H']                           = '1'
  d['HAVE_CHSIZE']                                 = '1'
  d['HAVE_CRTDEFS_H']                              = if_msvc()
  d['HAVE_RAISE']                                  = '1'
  d['HAVE_READLINK']                               = '0'
  d['HAVE_WORKING_O_NOFOLLOW']                     = '0'
  d['HAVE__FTIME']                                 = '1'
  d['HAVE_SYS_TIMEB_H']                            = '1'
  d['HAVE_SYMLINK']                                = '0'
  d['HAVE_FCNTL']                                  = '1'
  d['HAVE_FNMATCH']                                = '0'
  d['HAVE_FNMATCH_H']                              = '0'
  d['HAVE_FCHOWNAT']                               = '0'
  d['HAVE_OPENAT']                                 = '0'
  d['HAVE_INITSTATE']                              = '1'
  d['HAVE_LONG_LONG_INT']                          = '1'
  d['HAVE_UNSIGNED_LONG_LONG_INT']                 = '1'
  d['HAVE_ACOSF']                                  = if_mingw_or_msvc18()
  d['HAVE_ACOSL']                                  = if_mingw()
  d['HAVE_ALPHASORT']                              = '0'
  d['HAVE_ARPA_INET_H']                            = '0'
  d['HAVE_ASINF']                                  = if_mingw()
  d['HAVE_ASINL']                                  = if_mingw()
  d['HAVE_ATAN2F']                                 = if_mingw()
  d['HAVE_ATANF']                                  = if_mingw()
  d['HAVE_ATANL']                                  = if_mingw()
  d['HAVE_ATOLL']                                  = if_mingw_or_msvc18()
  d['HAVE_BTOWC']                                  = '1'
  d['HAVE_CANONICALIZE_FILE_NAME']                 = '0'
  d['HAVE_CBRT']                                   = if_mingw_or_msvc18()
  d['HAVE_CHOWN']                                  = '0'
  d['HAVE_CLOSEDIR']                               = '0' # if_mingw()
  d['HAVE_COPYSIGN']                               = if_mingw_or_msvc18()
  d['HAVE_COPYSIGNL']                              = if_mingw_or_msvc18()
  d['HAVE_COSF']                                   = '1'
  d['HAVE_COSHF']                                  = '1'
  d['HAVE_COSL']                                   = '1'
  d['HAVE_DECL_ACOSL']                             = if_mingw()
  d['HAVE_DECL_ASINL']                             = if_mingw()
  d['HAVE_DECL_ATANL']                             = if_mingw()
  d['HAVE_DECL_CBRTF']                             = if_mingw_or_msvc18()
  d['HAVE_DECL_CBRTL']                             = if_mingw_or_msvc18()
  d['HAVE_DECL_CEILF']                             = '1'
  d['HAVE_DECL_CEILL']                             = '1'
  d['HAVE_DECL_COPYSIGNF']                         = if_mingw_or_msvc18()
  d['HAVE_DECL_COSL']                              = '1'
  d['HAVE_DECL_DIRFD']                             = '0'
  d['HAVE_DECL_ENVIRON']                           = '1'
  d['HAVE_DECL_EXP2']                              = if_mingw_or_msvc18()
  d['HAVE_DECL_EXP2F']                             = if_mingw_or_msvc18()
  d['HAVE_DECL_EXP2L']                             = if_mingw_or_msvc18()
  d['HAVE_DECL_EXPL']                              = '1'
  d['HAVE_DECL_EXPM1L']                            = if_msvc(1800)
  d['HAVE_DECL_FCHDIR']                            = '0'
  d['HAVE_DECL_FDATASYNC']                         = '0'
  d['HAVE_DECL_FDOPENDIR']                         = '0'
  d['HAVE_DECL_FLOORF']                            = '1'
  d['HAVE_DECL_FLOORL']                            = '1'
  d['HAVE_DECL_FPURGE']                            = '0'
  d['HAVE_DECL_FREEADDRINFO']                      = '1'
  d['HAVE_DECL_FREXPL']                            = '1'
  d['HAVE_DECL_FSEEKO']                            = '0'
  d['HAVE_DECL_FTELLO']                            = '0'
  d['HAVE_DECL_GAI_STRERROR']                      = '0'
  d['HAVE_DECL_GETADDRINFO']                       = '1'
  d['HAVE_DECL_GETDELIM']                          = '0'
  d['HAVE_DECL_GETDOMAINNAME']                     = '0'
  d['HAVE_DECL_GETLINE']                           = '0'
  d['HAVE_DECL_GETLOADAVG']                        = '0'
  d['HAVE_DECL_GETLOGIN']                          = '0'
  d['HAVE_DECL_GETLOGIN_R']                        = '0'
  d['HAVE_DECL_GETNAMEINFO']                       = '1'
  d['HAVE_DECL_GETPAGESIZE']                       = '0'
  d['HAVE_DECL_GETUSERSHELL']                      = '0'
  d['HAVE_DECL_IMAXABS']                           = '1'
  d['HAVE_DECL_IMAXDIV']                           = '1'
  d['HAVE_DECL_INET_NTOP']                         = '0'
  d['HAVE_DECL_INET_PTON']                         = '0'
  d['HAVE_DECL_INITSTATE']                         = '0'
  d['HAVE_DECL_LDEXPL']                            = '1'
  d['HAVE_DECL_LOCALTIME_R']                       = '0'
  d['HAVE_DECL_LOG10L']                            = '1'
  d['HAVE_DECL_LOG2']                              = if_mingw_or_msvc18()
  d['HAVE_DECL_LOG2F']                             = if_mingw_or_msvc18()
  d['HAVE_DECL_LOG2L']                             = if_mingw_or_msvc18()
  d['HAVE_DECL_LOGB']                              = if_mingw()
  d['HAVE_DECL_LOGL']                              = '1'
  d['HAVE_DECL_MEMMEM']                            = '0'
  d['HAVE_DECL_MEMRCHR']                           = '0'
  d['HAVE_DECL_OBSTACK_PRINTF']                    = '0'
  d['HAVE_DECL_REMAINDER']                         = if_mingw_or_msvc18()
  d['HAVE_DECL_REMAINDERL']                        = if_mingw_or_msvc18()
  d['HAVE_DECL_RINTF']                             = if_mingw_or_msvc18()
  d['HAVE_DECL_ROUND']                             = '1'
  d['HAVE_DECL_ROUNDF']                            = '1'
  d['HAVE_DECL_ROUNDL']                            = '1'
  d['HAVE_DECL_SETENV']                            = '1'
  d['HAVE_DECL_SETHOSTNAME']                       = '0'
  d['HAVE_DECL_SETSTATE']                          = '0'
  d['HAVE_DECL_SINL']                              = '1'
  d['HAVE_DECL_SNPRINTF']                          = '1'
  d['HAVE_DECL_SQRTL']                             = '1'
  d['HAVE_DECL_STRDUP']                            = '1'
  d['HAVE_DECL_STRERROR_R']                        = '0'
  d['HAVE_DECL_STRNCASECMP']                       = if_mingw()
  d['HAVE_DECL_STRNDUP']                           = '0'
  d['HAVE_DECL_STRNLEN']                           = '0'
  d['HAVE_DECL_STRSIGNAL']                         = '0'
  d['HAVE_DECL_STRTOK_R']                          = '0'
  d['HAVE_DECL_STRTOIMAX']                         = if_mingw_or_msvc18()
  d['HAVE_DECL_STRTOUMAX']                         = if_mingw_or_msvc18()
  d['HAVE_DECL_TANL']                              = '1'
  d['HAVE_DECL_TCGETSID']                          = '0'
  d['HAVE_DECL_TRUNC']                             = '1'
  d['HAVE_DECL_TRUNCF']                            = '1'
  d['HAVE_DECL_TRUNCL']                            = '1'
  d['HAVE_DECL_TRUNCATE']                          = if_mingw()
  d['HAVE_DECL_TTYNAME_R']                         = '0'
  d['HAVE_DECL_TZNAME']                            = '!' + if_msvc(1900)
  d['HAVE_DECL_UNSETENV']                          = '0'
  d['HAVE_DECL_VSNPRINTF']                         = '1'
  d['HAVE_DECL_WCTOB']                             = '1'
  d['HAVE_DECL_WCWIDTH']                           = '0'
  d['HAVE_DIRENT_H']                               = '0'
  d['HAVE_DPRINTF']                                = '1'
  d['HAVE_DUP2']                                   = '1'
  d['HAVE_DUP3']                                   = '0'
  d['HAVE_DUPLOCALE']                              = '0'
  d['HAVE_EUIDACCESS']                             = '0'
  d['HAVE_EXPF']                                   = '1'
  d['HAVE_EXPL']                                   = '1'
  d['HAVE_EXPM1']                                  = '1'
  d['HAVE_EXPM1F']                                 = '1'
  d['HAVE_EXPLICIT_BZERO']                         = '0'
  d['HAVE_FABSF']                                  = '1'
  d['HAVE_FABSL']                                  = '1'
  d['HAVE_FACCESSAT']                              = '0'
  d['HAVE_FCHDIR']                                 = '0'
# d['HAVE_FCHMOD']                                 = '0'
  d['HAVE_FCHMODAT']                               = '0'
  d['HAVE_FCHOWNAT']                               = '0'
  d['HAVE_FDATASYNC']                              = '0'
  d['HAVE_FDOPENDIR']                              = '0'
  d['HAVE_FEATURES_H']                             = '0'
  d['HAVE_FFS']                                    = '0'
  d['HAVE_FFSL']                                   = '0'
  d['HAVE_FFSLL']                                  = '0'
  d['HAVE_FLOCK']                                  = '0'
  d['HAVE_FMA']                                    = if_mingw()
  d['HAVE_FMAF']                                   = if_mingw()
  d['HAVE_FMAL']                                   = if_mingw()
  d['HAVE_FMODF']                                  = '1'
  d['HAVE_FMODL']                                  = '1'
  d['HAVE_FORKPTY']                                = '0'
  d['HAVE_FREXPF']                                 = '1'
  d['HAVE_FSTATAT']                                = '0'
  d['HAVE_FSYNC']                                  = '0'
  d['HAVE_FTRUNCATE']                              = if_mingw()
  d['HAVE_FUTIMENS']                               = '0'
  d['HAVE_GETDTABLESIZE']                          = '0'
  d['HAVE_GETGROUPS']                              = '0'
  d['HAVE_GETHOSTNAME']                            = '1'
  d['HAVE_GETLOGIN']                               = '0'
  d['HAVE_GETOPT_H']                               = if_mingw()
  d['HAVE_GETPAGESIZE']                            = '0'
  d['HAVE_GETPASS']                                = '0'
  d['HAVE_GETRUSAGE']                              = '0'
  d['HAVE_GETSUBOPT']                              = '0'
  d['HAVE_GETTIMEOFDAY']                           = if_mingw()
  d['HAVE_GRANTPT']                                = '0'
  d['HAVE_GROUP_MEMBER']                           = '0'
  d['HAVE_GLOB_H']                                 = '0'
  d['HAVE_HYPOTF']                                 = '1'
  d['HAVE_HYPOTL']                                 = '1'
  d['HAVE_ILOGB']                                  = if_mingw()
  d['HAVE_ILOGBF']                                 = if_mingw()
  d['HAVE_ILOGBL']                                 = if_mingw_or_msvc18()
  d['HAVE_INTTYPES_H']                             = if_mingw_or_msvc18()
  d['HAVE_IMAXDIV_T']                              = '1'
  d['HAVE_ISBLANK']                                = '1' # if_mingw_or_msvc18()
  d['HAVE_ISNAND']                                 = '0'
  d['HAVE_ISNANF']                                 = if_mingw()
  d['HAVE_ISNANL']                                 = if_mingw()
  d['HAVE_ISWCNTRL']                               = '1'
  d['HAVE_ISWBLANK']                               = '1'
  d['HAVE_LANGINFO_H']                             = '0'
  d['HAVE_LANGINFO_ALTMON']                        = '0'
  d['HAVE_LANGINFO_CODESET']                       = '0'
  d['HAVE_LANGINFO_ERA']                           = '0'
  d['HAVE_LANGINFO_T_FMT_AMPM']                    = '0'
  d['HAVE_LANGINFO_YESEXPR']                       = '0'
  d['HAVE_LCHMOD']                                 = '0'
  d['HAVE_LCHOWN']                                 = '0'
  d['HAVE_LDEXPF']                                 = '1'
  d['HAVE_LIBUTIL_H']                              = '0'
  d['HAVE_LINK']                                   = '0'
  d['HAVE_LINKAT']                                 = '0'
  d['HAVE_LOG10F']                                 = '1'
  d['HAVE_LOG10L']                                 = '1'
  d['HAVE_LOG1P']                                  = if_mingw()
  d['HAVE_LOG1PF']                                 = if_mingw()
  d['HAVE_LOG1PL']                                 = if_mingw()
  d['HAVE_LOGBF']                                  = if_mingw()
  d['HAVE_LOGBL']                                  = if_mingw()
  d['HAVE_LOGF']                                   = '1'
  d['HAVE_LOGL']                                   = '1'
  d['HAVE_LONG_LONG_INT']                          = '1'
  d['HAVE_LSTAT']                                  = '0'
  d['HAVE_MAX_ALIGN_T']                            = 'defined(__cplusplus) || ' + if_mingw64()
  d['HAVE_MBRLEN']                                 = '1'
  d['HAVE_MBRTOWC']                                = '1'
  d['HAVE_MBSINIT']                                = '1'
  d['HAVE_MBSLEN']                                 = '1'
  d['HAVE_MBSNRTOWCS']                             = '1'
  d['HAVE_MBSRTOWCS']                              = '1'
  d['HAVE_MEMCHR']                                 = '1'
  d['HAVE_MEMPCPY']                                = '0'
  d['HAVE_MKDIRAT']                                = '0'
  d['HAVE_MKDTEMP']                                = '0'
  d['HAVE_MKFIFO']                                 = '0'
  d['HAVE_MKFIFOAT']                               = '0'
  d['HAVE_MKNOD']                                  = '0'
  d['HAVE_MKNODAT']                                = '0'
  d['HAVE_MKOSTEMP']                               = '0'
  d['HAVE_MKOSTEMPS']                              = '0'
  d['HAVE_MKSTEMP']                                = '0'
  d['HAVE_MKSTEMPS']                               = '0'
  d['HAVE_MODFF']                                  = '1'
  d['HAVE_MODFL']                                  = '1'
  d['HAVE_MONETARY_H']                             = '0'
  d['HAVE_NANOSLEEP']                              = if_mingw64()
  d['HAVE_NETDB_H']                                = '0'
  d['HAVE_NETINET_IN_H']                           = '0'
  d['HAVE_NET_IF_H']                               = '0'
  d['HAVE_NL_LANGINFO']                            = '0'
  d['HAVE_OPENAT']                                 = '0'
  d['HAVE_OPENDIR']                                = '0' # if_mingw()
  d['HAVE_OPENPTY']                                = '0'
  d['HAVE_OS_H']                                   = '0'
  d['HAVE_PCLOSE']                                 = if_mingw()
  d['HAVE_PIPE2']                                  = '0'
  d['HAVE_PIPE']                                   = '0'
  d['HAVE_POLL']                                   = '0'
  d['HAVE_POLL_H']                                 = '0'
  d['HAVE_POPEN']                                  = if_mingw()
  d['HAVE_POSIX_OPENPT']                           = '0'
  d['HAVE_POSIX_SIGNALBLOCKING']                   = '0'
  d['HAVE_POSIX_SPAWN']                            = '0'
  d['HAVE_POSIX_SPAWNATTR_T']                      = '0'
  d['HAVE_POSIX_SPAWN_FILE_ACTIONS_T']             = '0'
  d['HAVE_POSIX_SPAWN_FILE_ACTIONS_ADDCHDIR']      = '0'
  d['HAVE_POWF']                                   = '1'
  d['HAVE_PREAD']                                  = '0'
  d['HAVE_PSELECT']                                = '0'
  d['HAVE_PTHREAD_H']                              = if_mingw64()
  d['HAVE_PTHREAD_SIGMASK']                        = if_mingw64()
  d['HAVE_PTHREAD_SPINLOCK_T']                     = if_mingw64()
  d['HAVE_PTHREAD_T']                              = if_mingw64()
  d['HAVE_PTSNAME']                                = '0'
  d['HAVE_PTSNAME_R']                              = '0'
  d['HAVE_PTY_H']                                  = '0'
  d['HAVE_PWRITE']                                 = '0'
  d['HAVE_RAISE']                                  = '1'
  d['HAVE_RANDOM']                                 = '0'
  d['HAVE_RANDOM_H']                               = '0'
  d['HAVE_RANDOM_R']                               = '0'
  d['HAVE_RAWMEMCHR']                              = '0'
  d['HAVE_READDIR']                                = if_mingw()
  d['HAVE_READLINK']                               = '0'
  d['HAVE_READLINKAT']                             = '0'
  d['HAVE_REALLOCARRAY']                           = '0'
  d['HAVE_REALPATH']                               = '0'
  d['HAVE_REMAINDER']                              = if_mingw_or_msvc18()
  d['HAVE_REMAINDERF']                             = if_mingw_or_msvc18()
  d['HAVE_REMAINDERL']                             = if_mingw_or_msvc18()
  d['HAVE_RENAMEAT']                               = '0'
  d['HAVE_REWINDDIR']                              = if_mingw()
  d['HAVE_RINT']                                   = if_mingw_or_msvc18()
  d['HAVE_RINTL']                                  = if_mingw_or_msvc18()
  d['HAVE_RPMATCH']                                = '0'
  d['HAVE_SA_FAMILY_T']                            = '0'
  d['HAVE_SCANDIR']                                = '0'
  d['HAVE_SECURE_GETENV']                          = '0'
  d['HAVE_SETHOSTNAME']                            = '0'
  d['HAVE_SIGACTION']                              = '0'
  d['HAVE_SIGHANDLER_T']                           = '1'
  d['HAVE_SIGINFO_T']                              = '0'
  d['HAVE_SIGSET_T']                               = if_mingw()
  d['HAVE_SINF']                                   = '1'
  d['HAVE_SINHF']                                  = '1'
  d['HAVE_SINL']                                   = '1'
  d['HAVE_SLEEP']                                  = '0'
  d['HAVE_SPAWN_H']                                = '0'
  d['HAVE_SQRTF']                                  = '1'
  d['HAVE_SQRTL']                                  = '1'
  d['HAVE_STPCPY']                                 = '0'
  d['HAVE_STPNCPY']                                = '0'
  d['HAVE_STRCASECMP']                             = if_mingw()
  d['HAVE_STRCASESTR']                             = '0'
  d['HAVE_STRCHRNUL']                              = '0'
  d['HAVE_STRFMON_L']                              = '0'
  d['HAVE_STRINGS_H']                              = if_mingw()
  d['HAVE_STRPBRK']                                = '1'
  d['HAVE_STRPTIME']                               = '0'
  d['HAVE_STRSEP']                                 = '0'
  d['HAVE_STRTOD']                                 = '1'
  d['HAVE_STRTOLL']                                = if_mingw_or_msvc18()
  d['HAVE_STRTOULL']                               = if_mingw_or_msvc18()
  d['HAVE_STRUCT_ADDRINFO']                        = '1'
  d['HAVE_STRUCT_RANDOM_DATA']                     = '0'
  d['HAVE_STRUCT_SIGACTION_SA_SIGACTION']          = '0'
  d['HAVE_STRUCT_SOCKADDR_STORAGE']                = '1'
  d['HAVE_STRUCT_SOCKADDR_STORAGE_SS_FAMILY']      = '1'
  d['HAVE_STRUCT_SCHED_PARAM']                     = '0'  # if_mingw()
  d['HAVE_STRUCT_TIMEVAL']                         = '1'
  d['HAVE_STRUCT_TMS']                             = '0'
  d['HAVE_STRUCT_UTSNAME']                         = '0'
  d['HAVE_STRVERSCMP']                             = '0'
  d['HAVE_SCHED_H']                                = '0'
  d['HAVE_SYMLINK']                                = '0'
  d['HAVE_SYMLINKAT']                              = '0'
  d['HAVE_SYSEXITS_H']                             = '0'
  d['HAVE_SYS_BITYPES_H']                          = '0'
  d['HAVE_SYS_CDEFS_H']                            = '0'
  d['HAVE_SYS_FILE_H']                             = '1'
  d['HAVE_SYS_IOCTL_H']                            = '0'
  d['HAVE_SYS_INTTYPES_H']                         = '0'
  d['HAVE_SYS_LOADAVG_H']                          = '0'
  d['HAVE_SYS_PARAM_H']                            = if_mingw()
  d['HAVE_SYS_RESOURCE_H']                         = '0'
  d['HAVE_SYS_SELECT_H']                           = '0'
  d['HAVE_SYS_SOCKET_H']                           = '0'
  d['HAVE_SYS_STAT_H']                             = '1'
  d['HAVE_SYS_TIMES_H']                            = '0'
  d['HAVE_SYS_TIME_H']                             = if_mingw()
  d['HAVE_SYS_TYPES_H']                            = '1'
  d['HAVE_SYS_UIO_H']                              = '0'
  d['HAVE_SYS_UTSNAME_H']                          = '0'
  d['HAVE_TANF']                                   = '1'
  d['HAVE_TANHF']                                  = '1'
  d['HAVE_TANL']                                   = '1'
  d['HAVE_TERMIOS_H']                              = '0'
  d['HAVE_TIMEGM']                                 = '0'
  d['HAVE_TIMES']                                  = '0'
  d['HAVE_TIMEZONE_T']                             = '0'
  d['HAVE_TYPE_VOLATILE_SIG_ATOMIC_T']             = '1'
  d['HAVE_TYPE_VISIT']                             = if_mingw()
  d['HAVE_TRUNCATE']                               = if_mingw()
  d['HAVE_TSEARCH']                                = '0'
  d['HAVE_TWALK']                                  = '0'
  d['HAVE_TZNAME']                                 = '!' + if_msvc(1900)
  d['HAVE_TZSET']                                  = '1'
  d['HAVE_UNAME']                                  = '0'
  d['HAVE_UNISTD_H']                               = '1'
  d['HAVE_UNLINKAT']                               = '0'
  d['HAVE_UNLOCKPT']                               = '0'
  d['HAVE_USLEEP']                                 = '0'   # Fake it.
  d['HAVE_UTIL_H']                                 = '0'
  d['HAVE_UTIME_H']                                = if_mingw()
  d['HAVE_UTIME']                                  = '1'
  d['HAVE_UTIMENSAT']                              = '0'
  d['HAVE_VASPRINTF']                              = '0'
  d['HAVE_VDPRINTF']                               = '0'
  d['HAVE_WCHAR_H']                                = '1'
  d['HAVE_WCHAR_T']                                = '1'
  d['HAVE_WCTYPE_T']                               = '1'
  d['HAVE_WCTYPE_H']                               = '1'
  d['HAVE_WCTRANS_T']                              = '1'
  d['HAVE_WCPCPY']                                 = '0'
  d['HAVE_WCPNCPY']                                = '0'
  d['HAVE_WCRTOMB']                                = '1'
  d['HAVE_WCSCASECMP']                             = '0'
  d['HAVE_WCSCAT']                                 = '1'
  d['HAVE_WCSCHR']                                 = '1'
  d['HAVE_WCSCMP']                                 = '1'
  d['HAVE_WCSCOLL']                                = '1'
  d['HAVE_WCSCPY']                                 = if_msvc()
  d['HAVE_WCSCSPN']                                = '1'
  d['HAVE_WCSDUP']                                 = '1'
  d['HAVE_WCSFTIME']                               = '1'
  d['HAVE_WCSLEN']                                 = '1'
  d['HAVE_WCSNCASECMP']                            = '0'
  d['HAVE_WCSNCAT']                                = if_msvc()
  d['HAVE_WCSNCMP']                                = if_msvc()
  d['HAVE_WCSNCPY']                                = if_msvc()
  d['HAVE_WCSNLEN']                                = if_msvc()
  d['HAVE_WCSNRTOMBS']                             = '1'
  d['HAVE_WCSPBRK']                                = '1'
  d['HAVE_WCSRCHR']                                = '1'
  d['HAVE_WCSRTOMBS']                              = '1'
  d['HAVE_WCSSPN']                                 = '1'
  d['HAVE_WCSSTR']                                 = '1'
  d['HAVE_WCSTOK']                                 = '1'
  d['HAVE_WCSWIDTH']                               = '0'
  d['HAVE_WCSXFRM']                                = '1'
  d['HAVE_WINSOCK2_H']                             = '1'
  d['HAVE_WINT_T']                                 = '1'
  d['HAVE_WMEMCHR']                                = '1'
  d['HAVE_WMEMCMP']                                = '1'
  d['HAVE_WMEMCPY']                                = '1'
  d['HAVE_WMEMMOVE']                               = '1'
  d['HAVE_WMEMSET']                                = '1'
  d['HAVE_WS2TCPIP_H']                             = '1'
  d['HAVE_XLOCALE_H']                              = '0'
  d['HAVE__BOOL']                                  = '1'
  d['HAVE__EXIT']                                  = if_msvc(1900)
  d['HAVE___INLINE']                               = if_msvc()
  d['ICONV_CONST']                                 = 'const'
  d['INCLUDE_NEXT']                                = ''
  d['INCLUDE_NEXT_AS_FIRST_DIRECTIVE']             = get_next_h()
  d['INT32_MAX_LT_INTMAX_MAX']                     = '1'
  d['INT64_MAX_EQ_LONG_MAX']                       = '0'
  d['NEXT_ARPA_INET_H']                            = get_next_h()
  d['NEXT_ASSERT_H']                               = get_next_h()
  d['NEXT_AS_FIRST_DIRECTIVE_MATH_H']              = ''
  d['NEXT_CTYPE_H']                                = get_next_h()
  d['NEXT_DIRENT_H']                               = get_next_h (0)
  d['NEXT_ERRNO_H']                                = get_next_h()
  d['NEXT_FCNTL_H']                                = get_next_h()
  d['NEXT_FNMATCH_H']                              = no_such_h_file()
  d['NEXT_GETOPT_H']                               = get_next_h (0)
  d['NEXT_GLOB_H']                                 = no_such_h_file()
  d['NEXT_ICONV_H']                                = get_next_iconv_h()
  d['NEXT_INTTYPES_H']                             = get_next_inttypes_h()
  d['NEXT_LANGINFO_H']                             = no_such_h_file()
  d['NEXT_LIMITS_H']                               = get_next_h()
  d['NEXT_LOCALE_H']                               = get_next_h()
  d['NEXT_MONETARY_H']                             = no_such_h_file()
  d['NEXT_NETINET_IN_H']                           = no_such_h_file()
  d['NEXT_NET_IF_H']                               = no_such_h_file()
  d['NEXT_NETDB_H']                                = no_such_h_file()
  d['NEXT_POLL_H']                                 = no_such_h_file()
  d['NEXT_PTHREAD_H']                              = get_next_h (0)
  d['NEXT_PTY_H']                                  = no_such_h_file()
  d['NEXT_SEARCH_H']                               = get_next_h()
  d['NEXT_SIGNAL_H']                               = get_next_h()
  d['NEXT_SCHED_H']                                = get_next_h(0)
  d['NEXT_SPAWN_H']                                = no_such_h_file()
  d['NEXT_STDARG_H']                               = get_next_h() + get_va_stuff()
  d['NEXT_STDDEF_H']                               = get_next_h()
  d['NEXT_STDIO_H']                                = get_next_h()
  d['NEXT_STDLIB_H']                               = get_next_h()
  d['NEXT_STDINT_H']                               = get_next_h()
  d['NEXT_STRINGS_H']                              = get_next_h (0)
  d['NEXT_STRING_H']                               = get_next_h()
  d['NEXT_SYSEXITS_H']                             = no_such_h_file()
  d['NEXT_SYS_FILE_H']                             = get_next_h (0)
  d['NEXT_SYS_IOCTL_H']                            = get_next_h (0)
  d['NEXT_SYS_RESOURCE_H']                         = no_such_h_file()
  d['NEXT_SYS_SELECT_H']                           = no_such_h_file()
  d['NEXT_SYS_SOCKET_H']                           = no_such_h_file()
  d['NEXT_SYS_STAT_H']                             = get_next_h()
  d['NEXT_SYS_TIMES_H']                            = get_next_h (0)
  d['NEXT_SYS_TIME_H']                             = get_next_h (0)
  d['NEXT_SYS_UIO_H']                              = no_such_h_file()
  d['NEXT_SYS_UTSNAME_H']                          = no_such_h_file()
  d['NEXT_SYS_WAIT_H']                             = no_such_h_file()
  d['NEXT_TIME_H']                                 = get_next_h()
  d['NEXT_TERMIOS_H']                              = no_such_h_file()
  d['NEXT_UNISTD_H']                               = get_next_h (0)
  d['NEXT_UTIME_H']                                = get_next_h (0)
  d['NEXT_WCHAR_H']                                = get_next_h() + undef_wchar_min_max()
  d['NEXT_WCTYPE_H']                               = get_next_h()
  d['PRAGMA_COLUMNS']                              = ''
  d['PRAGMA_SYSTEM_HEADER']                        = '#pragma GCC system_header'
  d['PRIPTR_PREFIX']                               = ''
  d['PRI_MACROS_BROKEN']                           = '0'
  d['PTHREAD_H_DEFINES_STRUCT_TIMESPEC']           = '0'
  d['REPLACE_ACOSF']                               = '0'
  d['REPLACE_ASINF']                               = '0'
  d['REPLACE_ATANF']                               = '0'
  d['REPLACE_ATAN2']                               = '0'
  d['REPLACE_ATAN2F']                              = '0'
  d['REPLACE_BTOWC']                               = '1'
  d['REPLACE_CALLOC']                              = '1'
  d['REPLACE_CANONICALIZE_FILE_NAME']              = '1'
  d['REPLACE_CBRTF']                               = '0'
  d['REPLACE_CBRTL']                               = '0'
  d['REPLACE_CEIL']                                = '1'
  d['REPLACE_CEILF']                               = '1'
  d['REPLACE_CEILL']                               = '1'
  d['REPLACE_CHOWN']                               = '1'
  d['REPLACE_CLOSE']                               = '1'
  d['REPLACE_CLOSEDIR']                            = if_mingw()
  d['REPLACE_COSF']                                = '0'
  d['REPLACE_COSHF']                               = '0'
  d['REPLACE_CTIME']                               = '1'
  d['REPLACE_DIRFD']                               = '1'
  d['REPLACE_DPRINTF']                             = '1'
  d['REPLACE_DUP2']                                = '1'
  d['REPLACE_DUP']                                 = '1'
  d['REPLACE_DUPLOCALE']                           = '1'
  d['REPLACE_EXP2']                                = '0'
  d['REPLACE_EXP2L']                               = '0'
  d['REPLACE_EXPF']                                = '0'
  d['REPLACE_EXPM1']                               = '0'
  d['REPLACE_EXPM1F']                              = '0'
  d['REPLACE_FABSL']                               = '0'
  d['REPLACE_FACCESSAT']                           = '1'
  d['REPLACE_FCHOWNAT']                            = '0'
  d['REPLACE_FCLOSE']                              = '1'  # '!' + if_msvc(1900)
  d['REPLACE_FCNTL']                               = '1'
  d['REPLACE_FDOPEN']                              = '1'
  d['REPLACE_FDOPENDIR']                           = '0'
  d['REPLACE_FFLUSH']                              = '1'  # '!' + if_msvc(1900)
  d['REPLACE_FLOOR']                               = '0'
  d['REPLACE_FLOORF']                              = '0'
  d['REPLACE_FLOORL']                              = '0'
  d['REPLACE_FMA']                                 = '1'
  d['REPLACE_FMAF']                                = '1'
  d['REPLACE_FMAL']                                = '1'
  d['REPLACE_FMOD']                                = '0'
  d['REPLACE_FMODF']                               = '0'
  d['REPLACE_FMODL']                               = '0'
  d['REPLACE_FNMATCH']                             = '1'
  d['REPLACE_FOPEN']                               = '1'
  d['REPLACE_FORKPTY']                             = '0'
  d['REPLACE_FPRINTF']                             = '1'
  d['REPLACE_FPURGE']                              = '1'
  d['REPLACE_FREOPEN']                             = '1'
  d['REPLACE_FREXP']                               = '0'
  d['REPLACE_FREXPF']                              = '0'
  d['REPLACE_FREXPL']                              = '0'
  d['REPLACE_FSEEK']                               = '1'   # '!' + if_msvc(1900)
  d['REPLACE_FSEEKO']                              = '1'   # '!' + if_msvc(1900)
  d['REPLACE_FSTAT']                               = '1'
  d['REPLACE_FSTATAT']                             = '1'
  d['REPLACE_FTELL']                               = '1'
  d['REPLACE_FTELLO']                              = '1'
  d['REPLACE_FTRUNCATE']                           = '1'
  d['REPLACE_FUTIMENS']                            = '1'
  d['REPLACE_GAI_STRERROR']                        = '1'
  d['REPLACE_GETCWD']                              = '1'
  d['REPLACE_GETDELIM']                            = '1'
  d['REPLACE_GETDOMAINNAME']                       = '1'
  d['REPLACE_GETDTABLESIZE']                       = '1'
  d['REPLACE_GETGROUPS']                           = '1'
  d['REPLACE_GETLINE']                             = '1'
  d['REPLACE_GETLOGIN_R']                          = '1'
  d['REPLACE_GETPAGESIZE']                         = '1'
  d['REPLACE_GETPASS']                             = '1'
  d['REPLACE_GETTIMEOFDAY']                        = '1'
  d['REPLACE_GLOB']                                = '1'
  d['REPLACE_GLOB_PATTERN_P']                      = '1'
  d['REPLACE_GMTIME']                              = '1'
  d['REPLACE_HUGE_VAL']                            = '0'
  d['REPLACE_HYPOT']                               = '0'
  d['REPLACE_HYPOTF']                              = '0'
  d['REPLACE_HYPOTL']                              = '0'
  d['REPLACE_ICONV']                               = replace_iconv()
  d['REPLACE_ICONV_OPEN']                          = replace_iconv()
  d['REPLACE_ICONV_UTF']                           = replace_iconv()
  d['REPLACE_ILOGB']                               = '1'
  d['REPLACE_ILOGBL']                              = '1'
  d['REPLACE_ILOGBF']                              = '1'
  d['REPLACE_INET_NTOP']                           = '1'
  d['REPLACE_INET_PTON']                           = '1'
  d['REPLACE_IOCTL']                               = '1'
  d['REPLACE_ISATTY']                              = '1'
  d['REPLACE_ISFINITE']                            = '!' + if_msvc(1800)
  d['REPLACE_ISINF']                               = '1'
  d['REPLACE_ISNAN']                               = '1'
  d['REPLACE_ISWCNTRL']                            = '!' + if_mingw()
  d['REPLACE_ISWBLANK']                            = '1'
  d['REPLACE_ITOLD']                               = '1'
  d['REPLACE_LCHOWN']                              = '1'
  d['REPLACE_LDEXPL']                              = '0'
  d['REPLACE_LINK']                                = '1'
  d['REPLACE_LINKAT']                              = '1'
  d['REPLACE_LOCALECONV']                          = '1'
  d['REPLACE_LOCALTIME']                           = '1'
  d['REPLACE_LOCALTIME_R']                         = '1'
  d['REPLACE_LOG10']                               = '0'
  d['REPLACE_LOG10F']                              = '0'
  d['REPLACE_LOG10L']                              = '0'
  d['REPLACE_LOG1P']                               = '1'
  d['REPLACE_LOG1PF']                              = '1'
  d['REPLACE_LOG1PL']                              = '1'
  d['REPLACE_LOG2']                                = '1'
  d['REPLACE_LOG2F']                               = '1'
  d['REPLACE_LOG2L']                               = '1'
  d['REPLACE_LOG']                                 = '0'
  d['REPLACE_LOGB']                                = '1'
  d['REPLACE_LOGBF']                               = '1'
  d['REPLACE_LOGBL']                               = '1'
  d['REPLACE_LOGF']                                = '0'
  d['REPLACE_LOGL']                                = '0'
  d['REPLACE_LSEEK']                               = '1'
  d['REPLACE_LSTAT']                               = '1'
  d['REPLACE_MALLOC']                              = '1'
  d['REPLACE_MBRLEN']                              = '1'
  d['REPLACE_MBRTOWC']                             = '1'
  d['REPLACE_MBSINIT']                             = '!' + if_msvc(1900)
  d['REPLACE_MBSNRTOWCS']                          = '1'
  d['REPLACE_MBSRTOWCS']                           = '1'
  d['REPLACE_MBSTATE_T']                           = '0'
  d['REPLACE_MBTOWC']                              = '1'
  d['REPLACE_MEMCHR']                              = '1'
  d['REPLACE_MEMMEM']                              = '0'
  d['REPLACE_MKDIR']                               = '1'
  d['REPLACE_MKFIFO']                              = '1'
  d['REPLACE_MKNOD']                               = '1'
  d['REPLACE_MKSTEMP']                             = '1'
  d['REPLACE_MKTIME']                              = '1'
  d['REPLACE_MODF']                                = '0'
  d['REPLACE_MODFF']                               = '0'
  d['REPLACE_MODFL']                               = '0'
  d['REPLACE_NAN']                                 = '0'
  d['REPLACE_NANOSLEEP']                           = '!' + if_mingw64()
  d['REPLACE_NL_LANGINFO']                         = '1'
  d['REPLACE_NULL']                                = '0'
  d['REPLACE_OBSTACK_PRINTF']                      = '1'
  d['REPLACE_OPEN']                                = '1'
  d['REPLACE_OPENAT']                              = '1'
  d['REPLACE_OPENDIR']                             = if_mingw()
  d['REPLACE_OPENPTY']                             = '0'
  d['REPLACE_PERROR']                              = '1'
  d['REPLACE_POLL']                                = '1'
  d['REPLACE_POPEN']                               = '1'
  d['REPLACE_POSIX_SPAWN']                         = '0'
  d['REPLACE_POSIX_SPAWNP']                        = '0'
  d['REPLACE_POSIX_SPAWN_FILE_ACTIONS_ADDCHDIR']   = '0'
  d['REPLACE_POSIX_SPAWN_FILE_ACTIONS_ADDCLOSE']   = '0'
  d['REPLACE_POSIX_SPAWN_FILE_ACTIONS_ADDDUP2']    = '0'
  d['REPLACE_POSIX_SPAWN_FILE_ACTIONS_ADDOPEN']    = '0'
  d['REPLACE_PREAD']                               = '1'
  d['REPLACE_PRINTF']                              = '1'
  d['REPLACE_PSELECT']                             = '0'
  d['REPLACE_PTHREAD_SIGMASK']                     = '!' + if_mingw64()
  d['REPLACE_PTSNAME']                             = '1'
  d['REPLACE_PTSNAME_R']                           = '1'
  d['REPLACE_PUTENV']                              = '1'
  d['REPLACE_PWRITE']                              = '1'
  d['REPLACE_QSORT_R']                             = '1'
  d['REPLACE_RAISE']                               = '1'
  d['REPLACE_RANDOM_R']                            = '1'
  d['REPLACE_READ']                                = '1'
  d['REPLACE_READLINK']                            = '1'
  d['REPLACE_READLINKAT']                          = '1'
  d['REPLACE_REALLOC']                             = '1'
  d['REPLACE_REALPATH']                            = '1'
  d['REPLACE_REMAINDER']                           = '1'
  d['REPLACE_REMAINDERF']                          = '1'
  d['REPLACE_REMAINDERL']                          = '1'
  d['REPLACE_REMOVE']                              = '1' # '!' + if_msvc(1900)
  d['REPLACE_RENAME']                              = '1' # '!' + if_msvc(1900)
  d['REPLACE_RENAMEAT']                            = '1'
  d['REPLACE_RMDIR']                               = '1'
  d['REPLACE_ROUND']                               = '1'
  d['REPLACE_ROUNDF']                              = '1'
  d['REPLACE_ROUNDL']                              = '1'
  d['REPLACE_SELECT']                              = '1'
  d['REPLACE_SETENV']                              = '1'
  d['REPLACE_SETLOCALE']                           = '1'
  d['REPLACE_SIGNBIT']                             = '1 /* @REPLACE_SIGNBIT@ */'
  d['REPLACE_SIGNBIT_USING_GCC']                   = if_mingw()
  d['REPLACE_SINF']                                = '0'
  d['REPLACE_SINHF']                               = '0'
  d['REPLACE_SLEEP']                               = '0'
  d['REPLACE_SNPRINTF']                            = '1'
  d['REPLACE_SPRINTF']                             = '1'
  d['REPLACE_SQRTF']                               = '0'
  d['REPLACE_SQRTL']                               = '0'
  d['REPLACE_STAT']                                = '0'
  d['REPLACE_STDIO_READ_FUNCS']                    = '1'
  d['REPLACE_STDIO_WRITE_FUNCS']                   = '1'
  d['REPLACE_STPNCPY']                             = '1'
  d['REPLACE_STRCASESTR']                          = '1'
  d['REPLACE_STRCHRNUL']                           = '1'
  d['REPLACE_STRDUP']                              = '0'
  d['REPLACE_STRERROR']                            = '1'
  d['REPLACE_STRERROR_R']                          = '1'
  d['REPLACE_STRFMON_L']                           = '1'
  d['REPLACE_STRNCAT']                             = '0'
  d['REPLACE_STRNDUP']                             = '1'
  d['REPLACE_STRNLEN']                             = '1'
  d['REPLACE_STRSIGNAL']                           = '1'
  d['REPLACE_STRSTR']                              = '0'
  d['REPLACE_STRTOD']                              = '1'
  d['REPLACE_STRTOK_R']                            = '1'
  d['REPLACE_STRTOIMAX']                           = '0'
  d['REPLACE_STRTOUMAX']                           = '0'
  d['REPLACE_STRUCT_LCONV']                        = '1'
  d['REPLACE_STRUCT_TIMEVAL']                      = '1'
  d['REPLACE_STRFTIME']                            = '1'
  d['REPLACE_SYMLINK']                             = '1'
  d['REPLACE_SYMLINKAT']                           = '1'
  d['REPLACE_TANF']                                = '0'
  d['REPLACE_TANHF']                               = '0'
  d['REPLACE_TIMEGM']                              = '1'
  d['REPLACE_TOWLOWER']                            = '!' + if_mingw()
  d['REPLACE_TMPFILE']                             = '1'
  d['REPLACE_TRUNC']                               = '0'
  d['REPLACE_TRUNCF']                              = '0'
  d['REPLACE_TRUNCL']                              = '0'
  d['REPLACE_TRUNCATE']                            = '1'
  d['REPLACE_TZSET']                               = '1'
  d['REPLACE_TSEARCH']                             = '1'
  d['REPLACE_TTYNAME_R']                           = '1'
  d['REPLACE_UNLINK']                              = '1'
  d['REPLACE_UNLINKAT']                            = '1'
  d['REPLACE_UNSETENV']                            = '1'
  d['REPLACE_USLEEP']                              = '1'
  d['REPLACE_UTIME']                               = '1'
  d['REPLACE_UTIMENSAT']                           = '1'
  d['REPLACE_VASPRINTF']                           = '1'
  d['REPLACE_VDPRINTF']                            = '1'
  d['REPLACE_VFPRINTF']                            = '1'
  d['REPLACE_VPRINTF']                             = '1'
  d['REPLACE_VSNPRINTF']                           = '1'
  d['REPLACE_VSPRINTF']                            = '1'
  d['REPLACE_WCRTOMB']                             = '1'
  d['REPLACE_WCSNRTOMBS']                          = '1'
  d['REPLACE_WCSRTOMBS']                           = '1'
  d['REPLACE_WCSWIDTH']                            = '1'
  d['REPLACE_WCSFTIME']                            = '1'
  d['REPLACE_WCTOB']                               = '1'
  d['REPLACE_WCTOMB']                              = '1'
  d['REPLACE_WCWIDTH']                             = '1'
  d['REPLACE_WRITE']                               = '1'
  d['SYS_IOCTL_H_HAVE_WINSOCK2_H']                 = '0'
  d['SYS_IOCTL_H_HAVE_WINSOCK2_H_AND_USE_SOCKETS'] = '0'
  d['SYS_TIME_H_DEFINES_STRUCT_TIMESPEC']          = '0'
  d['TIME_H_DEFINES_STRUCT_TIMESPEC']              = if_mingw(3,21)
  d['UNISTD_H_DEFINES_STRUCT_TIMESPEC']            = if_mingw(3,21)
  d['UINT32_MAX_LT_UINTMAX_MAX']                   = '1'
  d['UINT64_MAX_EQ_ULONG_MAX']                     = '0'
  d['UNDEFINE_STRTOK_R']                           = '0'
  d['UNISTD_H_HAVE_WINSOCK2_H']                    = '1'
  d['UNISTD_H_HAVE_WINSOCK2_H_AND_USE_SOCKETS']    = '0'
  d['WINDOWS_64_BIT_OFF_T']                        = '1'
  d['WINDOWS_64_BIT_ST_SIZE']                      = '1'
  d['WINDOWS_STAT_TIMESPEC']                       = '1'
  return d

main()

